import { Observable } from 'rxjs/Observable';
import Promise from 'dojo-shim/Promise';
import { ComposeFactory } from 'dojo-compose/compose';
import { Destroyable } from 'dojo-compose/mixins/createDestroyable';
export declare type StoreIndex = number | string;
export interface MemoryStorePragma {
    /**
     * The identity of the object
     */
    id?: StoreIndex;
    /**
     * Should the item be replaced if already exists.
     */
    replace?: boolean;
}
export interface MemoryStorePromise<T> extends Promise<T> {
    /**
     * Retrieve an object from the store based on the object's ID
     * @param id The ID of the object to retrieve
     */
    get(id: StoreIndex): MemoryStorePromise<T>;
    get(): MemoryStorePromise<Iterator<T>>;
    /**
     * Put an item in the object store.
     * @param item The item to put
     * @param options The pragma to use when putting the object
     */
    put(item: T, options?: MemoryStorePragma): MemoryStorePromise<T>;
    /**
     * Add an item to the object store.
     * @param add The item to add
     * @param options The pragma to use when adding the object
     */
    add(item: T, options?: MemoryStorePragma): MemoryStorePromise<T>;
    /**
     * Patch an object in the store by providing a partial object.  The result will be a promise
     * that resolves with the patched object.
     * @param partial The partial object to patch the existing object with
     * @param options The pragma to use when patching the object
     */
    patch(partial: any, options?: MemoryStorePragma): MemoryStorePromise<T>;
    /**
     * Remove an object from the store.
     * @param id The ID of the object to remove
     * @param item The object to remove
     */
    delete(id: StoreIndex): MemoryStorePromise<boolean>;
    delete(item: T): MemoryStorePromise<boolean>;
    /**
     * Set the stores objects to an array
     */
    fromArray(items: T[]): MemoryStorePromise<void>;
}
export interface MemoryStoreOptions<T extends Object> {
    /**
     * Any initial data that should populate the store
     */
    data?: T[];
    /**
     * The property of each object to use as the identity for the object
     */
    idProperty?: StoreIndex;
}
export declare const enum ChangeTypes {
    Add = 1,
    Put = 2,
    Patch = 3,
    Delete = 4,
}
export interface ChangeRecord<T extends Object> {
    type: ChangeTypes;
    id: StoreIndex;
    target?: T;
}
export interface MemoryStoreMixin<T extends Object> {
    /**
     * The property that determines the ID of the object (defaults to `id`)
     */
    idProperty: StoreIndex;
    /**
     * Retrieve an object from the store based on the object's ID
     * @param id The ID of the object to retrieve
     */
    get(id: StoreIndex): MemoryStorePromise<T>;
    get(): MemoryStorePromise<T>;
    /**
     * Observe an object, any subsequent changes to the object can also be observed via the observable
     * interface that is returned.  If the object is not present in the store, the observation will be
     * immediatly completed.  If the object is deleted from the store, the observation will be completed
     * @param id The ID of the object to observe
     */
    observe(id: StoreIndex): Observable<T>;
    observe(): Observable<ChangeRecord<T>>;
    /**
     * Put an item in the object store.
     * @param item The item to put
     * @param options The pragma to use when putting the object
     */
    put(item: T, options?: MemoryStorePragma): MemoryStorePromise<T>;
    /**
     * Add an item to the object store.
     * @param add The item to add
     * @param options The pragma to use when adding the object
     */
    add(item: T, options?: MemoryStorePragma): MemoryStorePromise<T>;
    /**
     * Patch an object in the store by providing a partial object.  The result will be a promise
     * that resolves with the patched object.
     * @param partial The partial object to patch the existing object with
     * @param options The pragma to use when patching the object
     */
    patch(partial: any, options?: MemoryStorePragma): MemoryStorePromise<T>;
    /**
     * Remove an object from the store.
     * @param id The ID of the object to remove
     * @param item The object to remove
     */
    delete(id: StoreIndex): MemoryStorePromise<boolean>;
    delete(item: T): MemoryStorePromise<boolean>;
    /**
     * Set the stores objects to an array
     */
    fromArray(items: T[]): MemoryStorePromise<void>;
}
export declare type MemoryStore<T extends Object> = MemoryStoreMixin<T> & Destroyable;
export interface MemoryStoreFactory extends ComposeFactory<MemoryStore<Object>, MemoryStoreOptions<Object>> {
    <T extends Object>(options?: MemoryStoreOptions<T>): MemoryStore<T>;
    /**
     * Creates a memory store from an array of objects
     * @params data The array of data to create the memory store from
     */
    fromArray<T extends Object>(data: T[]): MemoryStore<T>;
}
/**
 * Create a new instance of a MemoryStore
 */
declare const createMemoryStore: MemoryStoreFactory;
export default createMemoryStore;
